import hashlib
import uuid
from .k2cfg import K2


db = K2.db

class K2admin_menus(db.Model):
    __tablename__ = 'k2admin_menus'
    menuid = db.Column(db.String(75), primary_key=True)
    namemenu = db.Column(db.String(150), index=True)
    prevmenu = db.Column(db.String(150), index=True)
    ccount = db.Column(db.Integer)
    scriptrun = db.Column(db.Text)
    prevmenu_text = db.Column(db.String(150))
    module_name = db.Column(db.String(150), index=True)
    magazinsid = db.Column(db.String(75), index=True)
    active = db.Column(db.Integer, index=True)
    createuser = db.Column(db.String(75), index=True)
    createdate = db.Column(db.DateTime, index=True)
    updateuser = db.Column(db.String(75), index=True)
    updatedate = db.Column(db.DateTime, index=True)
    order_ins = db.Column(db.String(150), index=True)
    caption = db.Column(db.String(1024))
    url = db.Column(db.String(2048))
    del_ = db.Column(db.Integer, name='del')


    def __init__(self, **kwargs):
        self.menuid = hashlib.md5(str(uuid.uuid4()).encode()).hexdigest()
        super(K2admin_menus, self).__init__(**kwargs)


    def __repr__(self):
        return f"{self.namemenu}')"

    @classmethod
    def get_menu_count(cls, name_menu):
        try:
            count = cls.query.filter(cls.namemenu == name_menu).count()
            return count
        except Exception as e:
            #logging.error(f"Error {str(e)}")
            return None


    #@classmethod
    def get_menu_items_by_roles(self, roleid):
        menu_items_b = (
            db.session.query(K2admin_menus.namemenu)
            .join(K2admin_Menus_Prava, K2admin_Menus_Prava.menuid == K2admin_menus.menuid)
            .filter(K2admin_Menus_Prava.roleid == roleid, K2admin_Menus_Prava.active == 1)
            .all()
        )
        menu_items = [item[0] for item in menu_items_b]
        return menu_items

class K2admin_Menus_Prava(db.Model):
    __tablename__ = 'k2admin_menus_prava'
    pravrepid = db.Column(db.String(75), primary_key=True)
    menuid = db.Column(db.String(75), db.ForeignKey('k2admin_menus.menuid'))
    username = db.Column(db.String(150))
    r = db.Column(db.Integer)
    w = db.Column(db.Integer)
    i = db.Column(db.Integer)
    d = db.Column(db.Integer)
    c = db.Column(db.Integer)
    exp = db.Column(db.Integer)
    imp = db.Column(db.Integer)
    del_ = db.Column(db.Integer, name='del')
    settable = db.Column(db.Integer)
    cutpast = db.Column(db.Integer)
    enable = db.Column(db.Integer)
    magazinsid = db.Column(db.String(75))
    active = db.Column(db.Integer)
    createuser = db.Column(db.String(75))
    createdate = db.Column(db.DateTime)
    updateuser = db.Column(db.String(75))
    updatedate = db.Column(db.DateTime)
    order_ins = db.Column(db.Integer)
    roleid = db.Column(db.String(150))

    menu = db.relationship('K2admin_menus', backref='prava')

    def __init__(self, **kwargs):
        self.pravrepid = hashlib.md5(str(uuid.uuid4()).encode()).hexdigest()
        super(K2admin_Menus_Prava, self).__init__(**kwargs)

