import yaml
import inspect
import pytz
import os
import sys
import platform
import uuid

from flask import session, request, redirect, url_for
from flask_login import LoginManager, current_user
from sqlalchemy.orm import joinedload
import logging


from .k2obj import K2Obj
from .k2secur import K2Secur
from .k2path import K2Path

class K2(K2Obj):
    name = 'k2cfg'
    version = '2.5.1.41'
    port = ''
    domain_name = ''
    domain = ''
    update_domain = ''
    components_install = []
    menu = []
    menu_url = []
    menu_category = []
    sglalchemy_pool_size = 15
    sglalchemy_max_overflow = 30
    safe_mode = False
    json_sort_keys = False
    babel_translation_directories = ''
    default_language = 'uk'
    current_language = 'uk'
    login_manager = LoginManager()
    timezone = pytz.timezone('Europe/Kiev')
    secret_key = 'JH&INH987gFDHdsagh&8dwbjdw8ckw'
    update_token = ''
    generate_migration = ''
    upgrade_migrations = ''
    platform = ''
    user_ip = ''
    user_agent = ''
    languages = {
        'en': 'English',
        'uk': 'Українська',
        'pl': 'Polski'
    }
    proj_config = yaml.safe_load(open("../proj.yml"))['proj']
    db = ''
    db_driver = ''
    adm_menu_items = [
        {
            "title": "t_update_system",
            "icon": {"icon": "mdi mdi-cloud-download"},
            "children": [
                {
                    "title": "t_install_new_components2",
                    "to": "/components-add"
                },
                {
                    "title": "t_installed_components2",
                    "to": "/components-list"
                },
                {

                    "title": "t_update_component",
                    "to": "/components-update"

                }

            ]
        }
    ]
    adm_menu_items_category = [
        {
            "title": 't_update_system',  # назва модуля
            "to": "/components-add",  # посилання на головну сторінку модуля
            "icon": "k2",  # назва іконки модуля
            "sort": 100
        }
    ]

    # шляхи до файлу з підключенням до БД
    path_db = f"../cfg/{proj_config}/db"

    search_comp = [
        f"../cfg/{proj_config}/components",
        f"../usr/{proj_config}/components",
        f"components"
    ]
    app_root_path = ''
    template = 'app_stack'
    authorized_users = {}


    def __init__(self, *args, **kwargs):
        super().__init__()
        self.current_user_login = self.get_current_user()
        self.secur = K2Secur()
        self.path = K2Path()

        #self.menu = self.search_menu_items()


    def get_current_user(self):
        '''Select current_user
        K2().get_current_user()'''

        from app import app
        if hasattr(app, 'login_manager') and isinstance(app.login_manager, LoginManager):
            return current_user
        else:
            return None

    def add_authorized_users(self, user_id, login):
        '''Додати користувача до списку авторизованих'''
        K2.authorized_users[user_id] = login

    def dell_authorized_users(self, user_id):
        '''Видалити користувача зі списку авторизованих'''
        K2.authorized_users.pop(user_id, None)

    # def get_macadress_for_ip(self, ip):
    #     'Returns a list of MACs for interfaces that have given IP, returns None if not found'
    #     import netifaces as nif
    #     for i in nif.interfaces():
    #         addrs = nif.ifaddresses(i)
    #         try:
    #             if_mac = addrs[nif.AF_LINK][0]['addr']
    #             if_ip = addrs[nif.AF_INET][0]['addr']
    #         except Exception as e:
    #             if_mac = if_ip = None
    #         if if_ip == ip:
    #             return if_mac
    #     return None

    # load database settings from yml file
    def init_db_uri(self):
        '''Пошук і ініціалізація файлу з підключення до бази даних '''
        #for path in self.path_db:
        try:
            db_file_path = self.path_db + '/db.yml'
            if os.path.exists(db_file_path) and os.path.getsize(db_file_path) > 0:
                with open(db_file_path, 'r') as f:
                    db_config = yaml.load(f, Loader=yaml.FullLoader)
                return db_config
            return None
        except Exception as e:
            logging.error(f"{e}")
            return None


    # creating a connection
    def init_db(self):
        '''Підключення db_uri для вибраної бази даних'''
        db_config = self.init_db_uri()
        if db_config is None:
            db_path = 'database.db'
            db_uri = f"sqlite:///{db_path}"
        else:
            db_uri = f"{db_config['driver']}://{db_config['user']}:{db_config['password']}@{db_config['host']}:{db_config['port']}/{db_config['dbname']}"
        return db_uri
        # app.config['SQLALCHEMY_DATABASE_URI'] = db_uri
        # app.config['SQLALCHEMY_BINDS'] = {'db2': f'sqlite:///{db_path}'}
        # self.db.init_app(app)

    def init_lm(self, app):
        '''Ініціалізація логін-менеджера'''
        # Initialize LoginManager
        self.login_manager.init_app(app)



    def get_user_project_id(self):
        '''Пошук id поточного проекту користувача'''
        from app import app
        if hasattr(app, 'login_manager') and isinstance(app.login_manager,
                                                        LoginManager) and current_user.is_authenticated:
            from components.k2site.k2site.models import K2users
            user_id = current_user.get_id()
            user = K2users.query.get(user_id)
            if user is not None:
                current_proj_id = user.projid
            else:
                current_proj_id = None
        else:
            current_proj_id = None
        return current_proj_id

    def get_user_project_name(self):
        '''Пошук назви поточного проекту користувача'''
        try:
            if self.get_user_project_id() :
                from components.k2site.k2site.models import K2Proj
                projid = self.get_user_project_id()
                project = K2Proj.query.get(projid)
                if project:
                    projname = project.projname
                else:
                    projname = None
            else:
                projname = None
            return projname
        except Exception as e:
            logging.error(f"{e}")
            return None


    def get_user_counterparts_id(self):
        '''Пошук id поточного контрагента'''
        from app import app
        if hasattr(app, 'login_manager') and isinstance(app.login_manager,
                                                        LoginManager) and current_user.is_authenticated:
            from components.k2site.k2site.models import K2users
            user_id = current_user.get_id()
            user = K2users.query.get(user_id)
            if user is not None:
                current_counterpart_id = user.counterpart_id
            else:
                current_counterpart_id = None
        else:
            current_counterpart_id = None
        return current_counterpart_id

    def get_user_counterparts_name(self):
        '''Пошук назви поточного контрагента'''
        try:
            if self.get_user_counterparts_id() :
                from components.k2handbook.k2handbook.models import k2counterparts
                counterparts_id = self.get_user_counterparts_id()
                counterparts = k2counterparts.query.get(counterparts_id)
                if counterparts:
                    counterparts_name = counterparts.counterpart_name
                else:
                    counterparts_name = '-'
            else:
                counterparts_name = '-'
            return counterparts_name
        except Exception as e:
            logging.error(f"{e}")
            counterparts_name = '-'
            return counterparts_name


    def get_user_storage_id(self):
        '''Пошук id поточного складу'''
        from app import app
        if hasattr(app, 'login_manager') and isinstance(app.login_manager,
                                                        LoginManager) and current_user.is_authenticated:
            from components.k2site.k2site.models import K2users
            user_id = current_user.get_id()
            user = K2users.query.get(user_id)
            if user is not None:
                current_storage_id = user.storage_id
            else:
                current_storage_id = None
        else:
            current_storage_id = None
        return current_storage_id

    def get_user_stoages_name(self):
        '''Пошук назви поточного складу'''
        try:
            if self.get_user_storage_id() :
                from components.k2handbook.k2handbook.models import k2storage
                storage_id = self.get_user_storage_id()
                storages = k2storage.query.get(storage_id)
                if storages:
                    storages_name = storages.storage_name
                else:
                    storages_name = '-'
            else:
                storages_name = '-'
            return storages_name
        except Exception as e:
            logging.error(f"{e}")
            storages_name = '-'
            return storages_name


    def get_project_setting(self):
        '''Форма зміни проекту'''
        from app import app
        if hasattr(app, 'login_manager') and isinstance(app.login_manager,
                                                        LoginManager) and current_user.is_authenticated:
            try:
                from components.k2site.k2site.views import K2site
                grid_settings = K2site.show_projects()
                return grid_settings
            except Exception as e:
                logging.error(f"{e}")
                grid_settings = ''
                return grid_settings



    def search_menu_items(self):
        '''Search menu items'''
        try:
            from app import app
            if hasattr(app, 'login_manager') and isinstance(app.login_manager,
                                                            LoginManager) and current_user.is_authenticated:
                from k2.k2upd import K2Menu
                response = K2Menu().get_admin_menu()
                K2.menu = response
                K2.menu_url = K2Menu().get_admin_menu_url(response)
            else:
                K2.menu = []
                K2.menu_url = []
        except Exception as e:
            logging.error(f"search_menu_items {e}")




    def search_menu_items_category(self):
        '''Search menu items category'''
        try:
            from app import app
            if hasattr(app, 'login_manager') and isinstance(app.login_manager,
                                                            LoginManager) and current_user.is_authenticated:
                from k2.k2upd import K2Menu
                response = K2Menu().get_admin_menu_category()  # requests.get(url=f"{K2.domain_loc}/api/main-menu-category/", headers=request.headers)
                # if response.status_code == 200:
                K2.menu_category = response
                # else:
                #    self.menu_category = []
            else:
                K2.menu_category = []
        except Exception as e:
            logging.error(f"{e}")


    def current_template(self):
        '''Search current template'''
        current_template_bd = None
        from app import app
        if hasattr(app, 'login_manager') and isinstance(app.login_manager,
                                                        LoginManager) and current_user.is_authenticated:
            from components.k2site.k2site.models import K2users
            user_id = current_user.get_id()
            user = K2users.query.get(user_id)
            if user is not None:
                current_template_bd = user.user_theme_pyt
        if current_template_bd is not None:
            return current_template_bd
        else:
            return self.template

    # def get_user_agent(self, current_request):
    #     pass


    @classmethod
    def namemenu(cls, url):
        namemenu_itm = 'k2itm' + url.replace('/', '-')
        return namemenu_itm

    @classmethod
    def get_user_role(cls, user_id):
        '''Отрмання roleid користувача '''
        try:
            from app import app
            if hasattr(app, 'login_manager') and isinstance(app.login_manager,
                                                            LoginManager) and current_user.is_authenticated:
                from components.k2site.k2site.models import K2users
                from .k2admmenu import K2admin_Menus_Prava, K2admin_menus

            # Отримати roleid
            user_role = K2users.query.filter_by(user_id=user_id).first().roleid
            return user_role
        except Exception as ex:
             print(ex)
             return {'error': 'Помилка при виконанні запиту до бази даних'}



    @classmethod
    def get_user_permissions(cls, user_id, namemenu):
        '''Отрмання прав користувача залежно від url'''
        try:
            from app import app
            if hasattr(app, 'login_manager') and isinstance(app.login_manager,
                                                            LoginManager) and current_user.is_authenticated:
                from components.k2site.k2site.models import K2users
                from .k2admmenu import K2admin_Menus_Prava, K2admin_menus

            # Отримати roleid
            user_role = cls.get_user_role(user_id)

            # Отримати права за вказаним roleid та namemenu
            user_permissions = K2admin_Menus_Prava.query \
                .join(K2admin_menus, K2admin_Menus_Prava.menuid == K2admin_menus.menuid) \
                .filter(K2admin_Menus_Prava.roleid == user_role, K2admin_menus.namemenu == namemenu) \
                .options(joinedload(K2admin_Menus_Prava.menu)) \
                .all()

            # Перетворити результат в словник
            if user_permissions:
                permissions_list = []
                for permission in user_permissions:
                    permissions_dict = {
                        'roleid': permission.roleid,
                        'r': permission.r,
                        'w': permission.w,
                        'i': permission.i,
                        'd': permission.d,
                        'c': permission.c,
                        'exp': permission.exp,
                        'imp': permission.imp,
                        'del_': permission.del_,
                        'settable': permission.settable,
                        'cutpast': permission.cutpast,
                        'enable': permission.enable,
                        'active': permission.active
                        # Додайте інші поля за необхідності
                    }
                    permissions_list.append(permissions_dict)
            else:
                permissions_list = None

            return permissions_list[0]
        except Exception as ex:
             print(ex)
             return {'error': 'Помилка при виконанні запиту до бази даних'}

    @classmethod
    def search_yml(cls, name_yml):
        """Search for a yml file in the caller directories of subclasses.

        This method searches for a yml file with a specified name in the caller directories of all subclasses of K2Obj.
        The search is performed in the 'yml' subdirectory of each caller directory.

        Args:
            name_yml: The name of the yml file to search for (without the '.yml' extension).

        Returns:
            The caller directory where the yml file was found, or None if the file was not found.

        Usage:
            To search for a yml file named 'users.yml', call this method as follows:
                K2.search_yml('users')
        """
        for subclass in K2Obj.__subclasses__():
            caller_path = inspect.getfile(subclass)
            caller_dir = os.path.dirname(caller_path)

            # Search for the yml file in the 'yml' subdirectory
            yml_dir = os.path.join(caller_dir, 'yml')
            yml_file = os.path.join(yml_dir, f'{name_yml}.yml')
            if os.path.exists(yml_file):
                # Return the caller directory if the yml file is found
                return caller_dir

        # Return None if the yml file is not found
        return None

    @classmethod
    def get_path_to_root(cls, caller_file):
        '''
        Відносний шлях до рута.
        Виклик  K2.get_path_to_root(__file__).
        __file__ - стандартна змінна в Python, яка містить абсолютний шлях до поточного файлу, в якому вона використовується.
        '''
        curr_path = os.path.abspath('app.py')
        relative_path = os.path.relpath(caller_file, curr_path)
        return relative_path

    @classmethod
    def get_path_abs(cls, caller_file):
        '''
        Абсолютний шлях до файлу, в якому виконується код.
        Виклик K2.get_path_abs(__file__)
        __file__ -стандартна змінна в Python, яка містить абсолютний шлях до поточного файлу, в якому вона використовується. '''
        return os.path.abspath(caller_file)

    @classmethod
    def generate_id(cls):
        '''Генерація ID'''
        generated_id = uuid.uuid4().hex[:32]
        return ''.join([generated_id[i:i + 4] for i in range(0, len(generated_id), 4)])


    @classmethod
    def load_babel_translation_directories(cls):
        '''Завантаження перекладів'''
        with open('languages/babel_translation_directories.yml', 'r') as f:
            data = yaml.safe_load(f)
            # print(data['babel_translation_directories'])
        cls.babel_translation_directories = data['babel_translation_directories']


    @classmethod
    def search_babel_translation_directories(cls):
        '''Пошук файлів з перекладами'''
        # response = requests.get(f'{cls.domain}api/languages')
        from k2.k2upd import K2Lang
        response = K2Lang.find_language()
        if response:
            cls.babel_translation_directories = response
            data = {'babel_translation_directories': response}
        else:
            cls.babel_translation_directories = 'languages'
            data = {'babel_translation_directories': 'languages'}

        with open('languages/babel_translation_directories.yml', 'w') as f:
            yaml.dump(data, f)



    @classmethod
    def get_locale(cls):
        '''Визначення поточної мови'''
        if 'lang' in session:
            # Отримати обрану мову з сеансу користувача
            return session['lang']
        elif cls.current_language != '':
            return cls.current_language
        else:
            # Повернути мову за замовчуванням з налаштувань додатку
            return cls.default_language  # app.config['DEFAULT_LANGUAGE']

    @classmethod
    def get_locale_id(cls):
        '''Визначення id поточної мови'''
        lang = cls.get_locale()
        try:
            from components.k2handbook.k2handbook.models import k2language
            language = k2language.query.filter_by(language_symbol=lang).first()
            if language:
                lang_id = language.language_id
            else:
                lang_id = None
        except Exception as e:
            logging.error(f"Помилка при виконанні запиту до бази даних. {e}")
            lang_id = None

        return lang_id


    @classmethod
    def get_platform(cls):
        '''Визначення операційної системи'''
        operating_system = platform.system()
        cls.platform = operating_system
        return cls.platform

    path_objs = [
        f"../cfg/{proj_config}/k2/{name.lower()}.py",
        f"cfg/{name.lower()}.py",
        f"usr/cfg/{name.lower()}.py"
    ]

    @classmethod
    def ins_search_comp(self):
        '''Вивід списку компонент з файлу components.yml у всіх шляхах пошуку '''
        result_dict = {'components': []}
        for components_list_yml in self.search_comp:
            if os.path.exists(components_list_yml):
                with open(f"{components_list_yml}/components.yml", 'r') as file:
                    components_data_itm = yaml.safe_load(file)

                    for component in components_data_itm['components']:
                        name = component['name']
                        existing_component = next(
                            (c for c in result_dict['components'] if c['name'] == name),
                            None
                        )
                        if existing_component is None:
                            result_dict['components'].append(component)
        return result_dict

    # @staticmethod
    def component_list():
        '''Пошук встановлених компонент'''
        with open(f"components/components.yml", 'r') as file:
            components_data = yaml.safe_load(file)
        if components_data is None or 'components' not in components_data:
            components_d = []
        else:
            components_d = components_data['components']
        installed_components = [{component['id']: component['name']} for component in components_d if
                                component.get('installed', False)]
        return installed_components

    components_install = component_list()  # список встановлених компонент

    # Отримуємо значення з path_objs для розширення поточного класу
    # @staticmethod
    def search_class_prop(path_objs):
        ''''''
        code_str = None
        for path_obj in path_objs:
            if os.path.exists(path_obj):
                with open(path_obj, 'r') as file:
                    code_str = file.read()
                break  # Виходимо з циклу, якщо знайдено файл
        else:
            sys.exit()  # Завершуємо процес, якщо жоден файл не знайдено
        return code_str

    code_str = K2Obj.search_class_prop(path_objs)


    try:
        exec(code_str)
    except Exception as e:
        print(f"Error in K2cfg {e}")
        sys.exit()
    domain_loc = f"http://127.0.0.1:{port}"

# print(K2.__dict__)
# print(K2.path_objs)

