from flask_babel import *
from flask_babel import Babel, Domain
from flask.helpers import locked_cached_property
from typing import List, Callable, Optional, Union

class K2Domain(Domain):
    def get_translations(self):
        ctx = _get_current_context()

        if ctx is None:
            return support.NullTranslations()

        cache = self.get_translations_cache(ctx)
        locale = get_locale()
        try:
            cache.clear()
            self.cache.clear()

            return cache[str(locale), self.domain[0]]
        except KeyError:
            translations = support.Translations()

            for index, dirname in enumerate(self.translation_directories):

                domain = (
                    self.domain[0]
                    if len(self.domain) == 1
                    else self.domain[index]
                )

                catalog = support.Translations.load(
                    dirname,
                    [locale],
                    domain
                )
                translations.merge(catalog)
                # FIXME: Workaround for merge() being really, really stupid. It
                # does not copy _info, plural(), or any other instance variables
                # populated by GNUTranslations. We probably want to stop using
                # `support.Translations.merge` entirely.
                if hasattr(catalog, 'plural'):
                    translations.plural = catalog.plural

            cache[str(locale), self.domain[0]] = translations
            return translations

class K2Babel(Babel):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    @locked_cached_property
    def domain_instance(self):
        """The message domain for the translations.
        """
        return K2Domain(domain=self.domain)



def _get_current_context() -> Optional[SimpleNamespace]:
    if not g:
        return None

    if not hasattr(g, "_flask_babel"):
        g._flask_babel = SimpleNamespace()

    return g._flask_babel  # noqa


def get_domain() -> K2Domain:
    ctx = _get_current_context()
    if ctx is None:
        # this will use NullTranslations
        return K2Domain()

    try:
        return ctx.babel_domain
    except AttributeError:
        pass

    ctx.babel_domain = get_babel().instance.domain_instance
    return ctx.babel_domain


# Create shortcuts for the default Flask domain
def gettext(*args, **kwargs) -> str:
    return get_domain().gettext(*args, **kwargs)


_ = gettext


def ngettext(*args, **kwargs) -> str:
    return get_domain().ngettext(*args, **kwargs)


def pgettext(*args, **kwargs) -> str:
    return get_domain().pgettext(*args, **kwargs)


def npgettext(*args, **kwargs) -> str:
    return get_domain().npgettext(*args, **kwargs)


def lazy_gettext(*args, **kwargs) -> LazyString:
    return LazyString(gettext, *args, **kwargs)


def lazy_pgettext(*args, **kwargs) -> LazyString:
    return LazyString(pgettext, *args, **kwargs)


def lazy_ngettext(*args, **kwargs) -> LazyString:
    return LazyString(ngettext, *args, **kwargs)


def lazy_npgettext(*args, **kwargs) -> LazyString:
    return LazyString(npgettext, *args, **kwargs)

